import sys
import persistent
import zope.interface
import zope.component
import zope.location
import zope.publisher.browser
import zope.publisher.interfaces
import zope.publisher.interfaces.browser

import zc.vault.vault
import zc.vault.interfaces


class Versions(persistent.Persistent, zope.location.Location):
    """Sequence of capability family versions.

    Used to implement CapabilityFamily.versions
    """
    
    zope.interface.implements(zc.vault.interfaces.IVersions)

    def __init__(self, vault, factory, parent=None, name=None, initialize=None):
        self.vault = vault
        self.factory = factory
        if vault.__parent__ is None:
            zope.location.locate(self.vault, self, 'vault')
        elif parent is None:
            raise RuntimeError(
                "programmer error: Locate the vault, or pass a parent in, "
                "or both")
        if parent is not None:
            if name is not None:
                zope.location.locate(self, parent, name)
            else:
                self.__parent__ = parent
        for ix in range(len(vault)):
            i = vault.getInventory(ix)
            assert (
                not zc.vault.interfaces.IWrapperAware.providedBy(
                    i.manifest)), (
                'programmer error: manifests in vault have already been placed '
                'in a Versions container')
            i.__parent__ = self
            wrapper = self.factory(i)
            i.manifest.wrapper = wrapper
            zope.interface.directlyProvides(
                i.manifest, zc.vault.interfaces.IWrapperAware)
            if zope.location.interfaces.ILocation.providedBy(wrapper):
                zope.location.locate(wrapper, self, str(i.manifest.vault_index))
        if initialize is not None:
            res = self.create()
            initialize(res)
            self.commit(res)

    def __len__(self):
        return len(self.vault)

    def __getitem__(self, idx):
        manifest = self.vault[idx]
        return manifest.wrapper

    def __iter__(self):
        for m in self.vault:
            yield m.wrapper

    def commit(self, wrapper):
        manifest = wrapper.inventory.manifest # XXX currently .inventory is
        # undocumented, hard requirement of wrapper...
        assert manifest.wrapper is wrapper, (
            'programmer error: manifest should have back reference to '
            'version')
        self.vault.commit(manifest)
        if zope.location.interfaces.ILocation.providedBy(wrapper):
            zope.location.locate(wrapper, self, str(manifest.vault_index))

    def commitFrom(self, wrapper):
        manifest = wrapper.inventory.manifest
        assert manifest.wrapper is wrapper, (
            'programmer error: manifest should have back reference to '
            'version')
        self.vault.commitFrom(manifest)
        i = self.vault.getInventory(-1)
        wrapper = self.factory(i)
        i.manifest.wrapper = wrapper
        zope.interface.directlyProvides(
            i.manifest, zc.vault.interfaces.IWrapperAware)
        if zope.location.interfaces.ILocation.providedBy(wrapper):
            zope.location.locate(wrapper, self, str(i.manifest.vault_index))

    def create(self):
        inventory = zc.vault.vault.Inventory(vault=self.vault, mutable=True)
        inventory.__parent__ = self
        res = self.factory(inventory)
        inventory.manifest.wrapper = res
        zope.interface.directlyProvides(
            inventory.manifest, zc.vault.interfaces.IWrapperAware)
        res.__parent__ = self
        return res


class deferredProperty(object):
    def __init__(self, name, initialize):
        self.name = name
        sys._getframe(1).f_locals[name] = self
        self.initialize = initialize
    def __get__(self, obj, typ=None):
        if obj is not None:
            self.initialize(obj)
            return obj.__dict__[self.name]
        return self


class Traverser(zope.publisher.browser.BrowserView):
    zope.component.adapts(
        zc.vault.interfaces.IVersions,
        zope.publisher.interfaces.browser.IBrowserRequest)
    zope.interface.implements(
        zope.publisher.interfaces.browser.IBrowserPublisher)

    _default = 'index.html'

    def browserDefault(self, request):
        return self.context, (self._default, )

    def publishTraverse(self, request, name):
        try:
            ix = int(name)
        except ValueError:
            pass
        else:
            try:
                v = self.context[ix]
            except IndexError:
                name = self._default
            else:
                return v
        view = zope.component.queryMultiAdapter(
            (self.context, request), name=name)
        if view is not None:
            return view
        raise zope.publisher.interfaces.NotFound(self.context, name, request)
